/*	maze.cpp -- maze-solving
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#ifdef WIN
#include <stdio.h>
#include <stdlib.h>
#endif
#include "abc.h"
#include "debug.h"
#include "maze.h"
#ifndef WIN
#include "comm.h"
#endif

/* 8'x8' board, min 6" from edge, 6" spacing;
   grid would be 15x15 junctions (225)
*/

enum	// NOTE: order is assumed! (clockwise)
{
	dirMin = 0,
	dirN = dirMin,
	dirE,
	dirS,
	dirW,
	dirMax
};

#define TurnForDir(dirCur, dirNew) (((dirNew) + dirMax - (dirCur)) % dirMax)

enum
{
	mssExplore = 0,		// normal exploring
	mssRetrace,			// retracing (e.g. subsequent run)
	mssUnwind,			// returning from dead end
};
/*
retrace -- go on exploring path until it ends
unwind -- go to prev node until find unknown path (!simple node...)
*/

/* junction storage...
   - simple: only one dir left (other paths are dead/none)
   - non-simple: at least one unknown (in addition to exploring);
     also stores x, y location (shorts),
     low bit of each used to encode dir used to get there;
     non-simple paths used when checking for loops!
*/

/* grfjct (uchar) contents... */
#define fjctSimple		0x80 // set if this is a simple junction (just grfjct)
#define fjctPrevSimple	0x40 // whether prev is simple jct (used when rewinding)
#define grfjctPathLMask 0x30 // holds info about left path (see fpath values below)
#define grfjctPathCMask 0x0C
#define grfjctPathRMask 0x03
#define grfjctPathMask	0x03 // 2 bits for path
/* shift values */
#define shjctPathL	4	// number of bits to shift fjctPath flags
#define shjctPathC	2
#define shjctPathR	0

/* path flags -- will be set for L, R, center at each junction */
#define fpathPotential	0x01	// there's a path here that might lead to end
#define fpathTraveled	0x02	// we are (or were) on this path
/* combinations of the above indicating status of the path */
#define grfpathNone		0x00	// no path in this direction
#define grfpathUnknown	(fpathPotential)	// path still need to explore
#define grfpathExplore	(fpathPotential | fpathTraveled)	// path we're on now
#define grfpathDeadEnd	(fpathTraveled)		// checked it out; led nowhere

#define CbJct(fSimple) (fSimple ? sizeof(uchar) \
						: sizeof(uchar) + 2 * sizeof(short))

#define FSimpleJct(grfjct) ((grfjct) & fjctSimple)
#define FSimpleJctPrev(grfjct) ((grfjct) & fjctPrevSimple)
#define PgrfjctNext(pgrfjct) ((pgrfjct) + CbJct(FSimpleJct(*(pgrfjct))))
#define PgrfjctPrev(pgrfjct) ((pgrfjct) - CbJct(FSimpleJctPrev(*(pgrfjct))))

/* stuff for non-simple junctions... */
/* x, y location of junction */
#define PxJct(pgrfjct)	((short *)(pgrfjct + 1))
#define PyJct(pgrfjct)	((short *)(pgrfjct + 3))
/* direction traveling when entered junction originally */
#define DirEnter(pgrfjct) \
			(((*PxJct(pgrfjct) & 0x01) << 1) | (*PyJct(pgrfjct) & 0x01))

/* unknown assumed to be set from None */
#define SetUnknownPath(grfjct, shjctPath) \
				((grfjct) |= (grfpathUnknown << (shjctPath)))
#define SetUnknownPathL(grfjct)		SetUnknownPath(grfjct, shjctPathL)
#define SetUnknownPathC(grfjct)		SetUnknownPath(grfjct, shjctPathC)
#define SetUnknownPathR(grfjct)		SetUnknownPath(grfjct, shjctPathR)
/* explore assumed to be set from Unknown (or None) */
#define SetExplorePath(grfjct, shjctPath) \
				((grfjct) |= (grfpathExplore << (shjctPath)))
#define SetExplorePathL(grfjct)		SetExplorePath(grfjct, shjctPathL)
#define SetExplorePathC(grfjct)		SetExplorePath(grfjct, shjctPathC)
#define SetExplorePathR(grfjct)		SetExplorePath(grfjct, shjctPathR)
/* dead end assumed to be set from Explore */
#define SetDeadEndPath(grfjct, shjctPath) \
				((grfjct) &= ~(fpathPotential << (shjctPath)))
#define SetDeadEndPathL(grfjct)		SetDeadEndPath(grfjct, shjctPathL)
#define SetDeadEndPathC(grfjct)		SetDeadEndPath(grfjct, shjctPathC)
#define SetDeadEndPathR(grfjct)		SetDeadEndPath(grfjct, shjctPathR)
/* checking for unknown path */
#define FUnknownPath(grfjct, grfjctPathMask, shjctPath) \
				(((grfjct) & (grfjctPathMask)) == (grfpathUnknown << shjctPath))
#define FUnknownPathL(grfjct)	FUnknownPath(grfjct, grfjctPathLMask, shjctPathL)
#define FUnknownPathC(grfjct)	FUnknownPath(grfjct, grfjctPathCMask, shjctPathC)
#define FUnknownPathR(grfjct)	FUnknownPath(grfjct, grfjctPathRMask, shjctPathR)
/* checking for path we're exploring */
#define FExplorePath(grfjct, grfjctPathMask, shjctPath) \
				(((grfjct) & (grfjctPathMask)) == (grfpathExplore << shjctPath))
#define FExplorePathL(grfjct)	FExplorePath(grfjct, grfjctPathLMask, shjctPathL)
#define FExplorePathC(grfjct)	FExplorePath(grfjct, grfjctPathCMask, shjctPathC)
#define FExplorePathR(grfjct)	FExplorePath(grfjct, grfjctPathRMask, shjctPathR)

#ifdef OPTICAL_DIST
//#define dzTolerance 200	// how close is "the same" (for optical mouse)
#define dzTolerance 275	// how close is "the same" (for optical mouse)
#else // OPTICAL_DIST
#define dzTolerance 75	// how close is "the same" (for time/ticks)
#endif // OPTICAL_DIST

#ifndef WIN
extern uchar g_bHeap;
#endif

MAZE::MAZE()
{
	Reset();
}

void MAZE::Reset()
{
#ifdef WIN
	m_pgrfjctMin = m_pgrfjctCur = (uchar *)malloc(1024);
#else
	m_pgrfjctCur = &g_bHeap;	// start at beginning of available "heap" memory
#endif
	/* we're at beginning, simple junction with only option as exploring center */
	*m_pgrfjctCur = fjctSimple | (grfpathExplore << shjctPathC);
	m_mss = mssExplore;
	m_dir = dirN;
	m_x = m_y = 0;
}

uchar MAZE::TurnAtJct(ushort dist, uchar grfpath)
{
	uchar turn;

	/* should only have directions set */
	Assert((grfpath & ~(fpathL | fpathR | fpathC)) == 0);

	/* handle easy cases -- 0 or 1 option for leaving
	   (doesn't require jct) */
	switch (grfpath)
	{
	case 0x00:
		/* dead end */
		Assert(m_mss == mssExplore || m_mss == mssRetrace);
		m_mss = mssUnwind;
		/* (don't care about location when unwinding) */
		turn = turn180;
		goto LTurn;
	case fpathL:
		/* L turn only */
		UpdateLocation(dist);
		turn = turnL;
		goto LTurn;
	case fpathR:
		/* R turn only */
		UpdateLocation(dist);
		turn = turnR;
		goto LTurn;
	case fpathC:
		Assert(fFalse);
		return turnNone;
	}

	/* have at least 2 options */
	switch (m_mss)
	{
	case mssRetrace:
		/* take explored path if available (else start exploring) */
		if ((turn = TurnRetrace()) != turnMax)
			break;
		/* done retracing; start exploring! (we're at new junction) */
		m_mss = mssExplore;
		/* fall through to explore case... */
	case mssExplore:
		UpdateLocation(dist);
		/* see if looped... */
		if ((turn = TurnCheckLoop()) != turnMax)
			break;
		/* add new node */
		FAddJct(grfpath);
		/* pick direction */
		turn = TurnExplore();
		break;
	case mssUnwind:
		/* leaving dead end; see if back to junction w/ unexplored path(s) */
		/* when unwinding, m_pgrfjctCur is already pointing to where
		   we just arrived! */
		if (FSimpleJct(*m_pgrfjctCur))
		{
			/* there was only the one path, and it's a dead end;
			   keep unwinding! */
			turn = TurnUnwind();
			m_pgrfjctCur = PgrfjctPrev(m_pgrfjctCur);
			/* REVIEW: check & handle error case... */
#ifdef WIN
			Assert(m_pgrfjctCur > m_pgrfjctMin);	// better not be back at start!
#else
			Assert(m_pgrfjctCur > &g_bHeap);	// better not be back at start!
#endif
		}
		else
		{
			turn = TurnFinishUnwind();
		}
		break;
	default:
		Assert(fFalse);
		turn = turnNone;
		break;
	}

LTurn:
	ChangeDir(turn);
	return turn;

/* REVIEW: use grfpath to validate against *m_pgrfjctCur! */
}

void MAZE::UpdateLocation(ushort dist)
{
	switch (m_dir)
	{
	case dirN:
		m_y += dist;
		break;
	case dirS:
		m_y -= dist;
		break;
	case dirE:
		m_x += dist;
		break;
	case dirW:
		m_x -= dist;
		break;
	default:
		Assert(fFalse);
		break;
	}
}

/* returns direction we'd head if execute specified turn */
void MAZE::ChangeDir(uchar turn)
{
	m_dir = (m_dir + turn) % dirMax;
}

char MAZE::FAddJct(uchar grfpath)
{
	char f;

	f = FSimpleJct(*m_pgrfjctCur);
	m_pgrfjctCur = PgrfjctNext(m_pgrfjctCur);
	/* REVIEW: check for memory availability! (false return...) */
	*m_pgrfjctCur = 0;
	if (f)
		*m_pgrfjctCur |= fjctPrevSimple;
	if (grfpath & fpathL)
		SetUnknownPathL(*m_pgrfjctCur);
	if (grfpath & fpathC)
		SetUnknownPathC(*m_pgrfjctCur);
	if (grfpath & fpathR)
		SetUnknownPathR(*m_pgrfjctCur);
	/* use low bit of x, y to encode direction */
//	Assert(m_dir >= 0);	// (always true since uchar!)
	Assert(m_dir < 4);
	*PxJct(m_pgrfjctCur) = (m_x & ~0x01) | ((m_dir >> 1) & 0x01);
		// & 0x01 shouldn't be necessary, but just in case...
	*PyJct(m_pgrfjctCur) = (m_y & ~0x01) | (m_dir & 0x01);
	return fTrue;
}

/* choose a path to explore */
uchar MAZE::TurnExplore()
{
	Assert(m_mss == mssExplore);

	/* choose unknown path to follow (and mark it) */
	if (FUnknownPathC(*m_pgrfjctCur))
	{
		SetExplorePathC(*m_pgrfjctCur);
		return turnNone;
	}
	if (FUnknownPathL(*m_pgrfjctCur))
	{
		SetExplorePathL(*m_pgrfjctCur);
		return turnL;
	}
	if (FUnknownPathR(*m_pgrfjctCur))
	{
		SetExplorePathR(*m_pgrfjctCur);
		return turnR;
	}
	Assert(fFalse);
	return turnNone;
}

/* choose a path when retracing;
	- if there are no more junctions, return turnMax; caller must
		handle adding new junction
	- choose the explored path
	- if there isn't an explored path, but we've alread got this junction,
		choose an unexplored path (and switch to explore mode)
*/
uchar MAZE::TurnRetrace()
{
	uchar *pgrfjct;

	if ((pgrfjct = PgrfjctNext(m_pgrfjctCur)) >= m_pgrfjctMac)
		return turnMax;		// junction not in list, yet
	m_pgrfjctCur = pgrfjct;

	/* follow explored path (if exists) */
	if (FExplorePathC(*m_pgrfjctCur))
		return turnNone;
	if (FExplorePathL(*m_pgrfjctCur))
		return turnL;
	if (FExplorePathR(*m_pgrfjctCur))
		return turnR;
	/* (end of the previously explored path) */
	Assert(!FSimpleJct(*m_pgrfjctCur));
	m_x = *PxJct(m_pgrfjctCur);
	m_y = *PyJct(m_pgrfjctCur);
	m_mss = mssExplore;
	return TurnExplore();
}

/* we're back at (simple) junction & need to keep rewinding path */
uchar MAZE::TurnUnwind()
{
	Assert(FSimpleJct(*m_pgrfjctCur));
	if (FExplorePathC(*m_pgrfjctCur))
	{
		/* we went straight from here to dead end, so need to go straight
		   to rewind path */
		return turnNone;
	}
	if (FExplorePathL(*m_pgrfjctCur))
	{
		/* turned L on way out; turn R on way back! */
		return turnR;
	}
	if (FExplorePathR(*m_pgrfjctCur))
	{
		/* turned R on way out; turn L on way back! */
		return turnL;
	}
	Assert(fFalse);
	return turnNone;
}

/* we've unwound to this (non-simple) jct; set explored path to dead end'
   see if can make this simple */
uchar MAZE::TurnFinishUnwind()
{
	uchar turn;

	Assert(!FSimpleJct(*m_pgrfjctCur));

	/* load x, y from jct... (back to known checkpoint) */
	m_x = *PxJct(m_pgrfjctCur);
	m_y = *PyJct(m_pgrfjctCur);
	m_mss = mssExplore;

	/* mark explored path as dead end; set turn to put us on next
	   unexplored path */

	/* shouldn't have R as unexplored since we've still got unknown
	   and R is the last path we try! */
	Assert(!FExplorePathR(*m_pgrfjctCur));

	if (FExplorePathC(*m_pgrfjctCur))
	{
		/* mark C as dead end */
		SetDeadEndPathC(*m_pgrfjctCur);
		/* we went straight out; need to turn 180 to be facing orig way */
		/* (assert isn't true if unwinding for save to EEPROM since
		   we don't update dir as unwind... ) */
//		Assert((m_dir + turn180) % dirMax == *(m_pgrfjctCur + 1));
		/* see if can explore L */
		if (FUnknownPathL(*m_pgrfjctCur))
		{
			SetExplorePathL(*m_pgrfjctCur);
			if (!FUnknownPathR(*m_pgrfjctCur))
			{
				/* this is our last remaining option */
				*m_pgrfjctCur |= fjctSimple;
			}
			/* coming from opposite of original dir, so R will put us on L path */
			turn = turnR;
		}
		else
		{
			/* if L not avail, must have R! */
			Assert(FUnknownPathR(*m_pgrfjctCur));
			SetExplorePathR(*m_pgrfjctCur);
			/* this is our last remaining option */
			*m_pgrfjctCur |= fjctSimple;
			/* coming from opposite of original dir, so L will put us on R path */
			turn = turnL;
		}
	}
	else
	{
		/* if C not explored, it must be L */
		Assert(FExplorePathL(*m_pgrfjctCur));
		/* mark L as dead end */
		SetDeadEndPathL(*m_pgrfjctCur);
		/* orig turned left; now back on that side; another L now would put us
		   facing orig dir */
		/* (assert isn't true if unwinding for save to EEPROM since
		   we don't update dir as unwind... ) */
//		Assert((m_dir + turnL) % dirMax == *(m_pgrfjctCur + 1));
		/* we better have option on R (otherwise would have been simple jct!) */
		Assert(FUnknownPathR(*m_pgrfjctCur));
		SetExplorePathR(*m_pgrfjctCur);
		/* we're simple now! */
		*m_pgrfjctCur |= fjctSimple;
		/* relative to orig dir: came from L, need to go to R;
		   so, keep going straight! */
		turn = turnNone;
	}

	return turn;
}

/* see if we've looped back to a junction we've seen before; if so,
   we want to unwind, treating the path we just took like a dead end;
   returns turnMax if didn't loop */
uchar MAZE::TurnCheckLoop()
{
	uchar *pgrfjct;
	char fSimpleLoop;

	fSimpleLoop = fTrue;	// let's hope for the easy one!
	for (pgrfjct = m_pgrfjctCur; 
#ifdef WIN
		pgrfjct >= m_pgrfjctMin;
#else
		pgrfjct >= &g_bHeap;
#endif
		pgrfjct = PgrfjctPrev(pgrfjct))
	{
		/* must have non-simple junction (simple junctions don't
		   have any unknown paths left), at the right location */
		if (!FSimpleJct(*pgrfjct))
		{
			if (Abs(*PxJct(pgrfjct) - m_x) < dzTolerance
				&& Abs(*PyJct(pgrfjct) - m_y) < dzTolerance)
			{
				/* for us to come back to a known junction,
				   we have to have arrived on an unknown path */
				/* figure out what turn need from original entrance dir
				   to get to path we're on */
				switch ((m_dir + turn180 + turnMax - DirEnter(pgrfjct)) % turnMax)
				{
				case turnNone:
					Assert(fFalse);		// we explore straight first!
					break;
				case turnL:
					if (FUnknownPathL(*pgrfjct))
					{
						/* say we've explored it & found a dead end */
						SetExplorePathL(*pgrfjct);
						SetDeadEndPathL(*pgrfjct);
						/* we won't have unknown path at C since go
						   there first */
						Assert(!FUnknownPathC(*pgrfjct));
						if (fSimpleLoop)
						{
							/* we're back at the most recent non-simple
							   junction; we can just stay here! */
							if (FUnknownPathR(*pgrfjct))
							{
								/* still more to explore from this jct */
								m_pgrfjctCur = pgrfjct;
								/* need to mark original path out as dead end
								   (since following the loop is useless);
								   since we came in on L and R is unknown,
								   had to be C explored! */
								Assert(FExplorePathC(*pgrfjct));
								SetDeadEndPathC(*pgrfjct);
								/* going straight will put us from L to R */
								SetExplorePathR(*pgrfjct);
								return turnNone;
							}
							/* go back the way we originally entered,
							   acting like it was a simple dead end */
							/* set current jct to the next one we'll
							   hit on the way back */
							m_pgrfjctCur = PgrfjctPrev(pgrfjct);
							m_mss = mssUnwind;
							/* if we came in on L, need to go R to leave */
							return turnR;
						}
						/* take care of fixing jct if it became simple!
						   (center won't be unknown since we go there first) */
						if (!FUnknownPathR(*pgrfjct))
							FixSimpleJct(pgrfjct);
						m_mss = mssUnwind;
						return turn180Special;
					}
					break;
				case turnR:
					if (FUnknownPathR(*pgrfjct))
					{
						/* say we've explored it & found a dead end */
						SetExplorePathR(*pgrfjct);
						SetDeadEndPathR(*pgrfjct);
						Assert(!FUnknownPathC(*pgrfjct));
						if (fSimpleLoop)
						{
							if (FUnknownPathL(*pgrfjct))
							{
								/* still more to explore from this jct */
								m_pgrfjctCur = pgrfjct;
								Assert(FExplorePathC(*pgrfjct));
								SetDeadEndPathC(*pgrfjct);
								/* going straight will put us from R to L */
								SetExplorePathL(*pgrfjct);
								return turnNone;
							}
							m_pgrfjctCur = PgrfjctPrev(pgrfjct);
							m_mss = mssUnwind;
							/* if we came in on R, need to go L to leave */
							return turnL;
						}
						/* take care of fixing jct if it became simple!
						   (center won't be unknown since we go there first) */
						if (!FUnknownPathL(*pgrfjct))
							FixSimpleJct(pgrfjct);
						m_mss = mssUnwind;
						return turn180Special;
					}
					break;
				case turn180:
					/* doesn't match */
					break;
				}
			}
			/* found non-simple junction, so this loop has
			   other stuff to explore */
			fSimpleLoop = fFalse;
		}
	}
	return turnMax;	// didn't find loop
}

/* convert jct to simple jct; it's in the middle of the memory
   so need to shift bytes over */
void MAZE::FixSimpleJct(uchar *pgrfjct)
{
	uchar *pgrfjctSrc;
	uchar *pgrfjctLim;

	Assert(!FUnknownPathL(*pgrfjct));
	Assert(!FUnknownPathC(*pgrfjct));
	Assert(!FUnknownPathR(*pgrfjct));

	Assert(!FSimpleJct(*pgrfjct));
	Assert(pgrfjct < m_pgrfjctCur);	// otherwise would have been simple loop

	/* find location of next jct (before we simplify this one!) */
	pgrfjctSrc = PgrfjctNext(pgrfjct);
	*pgrfjct |= fjctSimple;
	*pgrfjctSrc |= fjctPrevSimple;
	/* get spot where next jct belongs now... */
	pgrfjct = PgrfjctNext(pgrfjct);
	/* shift bytes over */
	while (pgrfjctSrc < m_pgrfjctCur)
		*pgrfjct++ = *pgrfjctSrc++;
	/* pgrfjctSrc is now at m_pgrfjctCur; pgrfjct is where it will be moved */
	pgrfjctLim = PgrfjctNext(m_pgrfjctCur);
	m_pgrfjctCur = pgrfjct;
	/* move current one */
	do
		*pgrfjct++ = *pgrfjctSrc++;
	while (pgrfjctSrc < pgrfjctLim);
}


/* had to stop exploring; clean up jcts and store to EEPROM */
#ifdef WIN
void MAZE::CleanupAndSave()
#else
void MAZE::CleanupAndSave(const EEPROM &eeprom)
#endif
{
#ifndef WIN
	uchar e;
	ushort cb;
	uchar *pgrfjct;
#endif

	if (m_mss == mssUnwind)
	{
		/* we were leaving dead end */
		/* keep backing up until get to non-simple node
		   (last place where there was a decision) */
		while (FSimpleJct(*m_pgrfjctCur))
		{
			m_pgrfjctCur = PgrfjctPrev(m_pgrfjctCur);
			/* REVIEW: check & handle error case... */
#ifdef WIN
			Assert(m_pgrfjctCur > m_pgrfjctMin);	// better not be back at start!
#else
			Assert(m_pgrfjctCur > &g_bHeap);	// better not be back at start!
#endif
		}
		/* mark dead end */
		(void)TurnFinishUnwind();
		/* note: this marked that we were exploring the new path; not
		   really true, but it works... */
	}

#ifndef WIN
	/* store to EEPROM */
	cb = PgrfjctNext(m_pgrfjctCur) - &g_bHeap;
	{
		COMM comm;
		comm.TxSz("Maze: ");
		comm.TxNum(cb);
		comm.TxCRLF();
	}
	eeprom.Write(ecblMaze, (uchar)(cb & 0xff));
	eeprom.Write(ecbhMaze, (uchar)(cb >> 8));
	e = ebMaze;
	for (pgrfjct = &g_bHeap; pgrfjct < PgrfjctNext(m_pgrfjctCur); ++pgrfjct, ++e)
		eeprom.Write(e, *pgrfjct);
#endif
}


#ifndef WIN
/* load maze info from EEPROM; if anything there,
   sets up to retrace from the beginning */
void MAZE::Load(const EEPROM &eeprom)
{
	uchar e;
	ushort cb;
	uchar *pgrfjct;

	cb = (ushort)(eeprom.Read(ecbhMaze) << 8) | eeprom.Read(ecblMaze);
	if (cb == 0 || cb > 512)	// REVIEW (512 is for error handling)
		return;

	Reset();	// REVIEW: always do this?!

	m_pgrfjctMac = m_pgrfjctCur + cb;
	e = ebMaze;
	for (pgrfjct = m_pgrfjctCur; pgrfjct < m_pgrfjctMac; ++pgrfjct, ++e)
		*pgrfjct = eeprom.Read(e);

	m_mss = mssRetrace;
}

void MAZE::Clear(const EEPROM &eeprom)
{
	eeprom.Write(ecblMaze, 0);
	eeprom.Write(ecbhMaze, 0);
}
#endif


#ifdef OLD
#ifdef DEBUG
void MAZE::Show() const
{
	uchar jct;
	const uchar *mpdirjct;
#ifndef WIN
	COMM comm;
#endif

	for (jct = 0; jct < m_jctMac; ++jct)
	{
		mpdirjct = m_dnjct[jct].mpdirjct;
#ifdef WIN
		printf("%d, %d", m_dnjct[jct].x, m_dnjct[jct].y);
		if (jct == m_jct)
			printf(" *");
		printf("\n");

		printf("         ");
		ShowJct(mpdirjct[dirN]);
		printf("\n");

		printf("     ");
		ShowJct(mpdirjct[dirW]);
		printf(" ");
		ShowJct(jct);
		printf(" ");
		ShowJct(mpdirjct[dirE]);
		printf("\n");

		printf("         ");
		ShowJct(mpdirjct[dirS]);
		printf("\n");
#else
		comm.TxNumSigned(m_dnjct[jct].x);
		comm.TxSz(", ");
		comm.TxNumSigned(m_dnjct[jct].y);
		if (jct == m_jct)
			comm.TxSz(" *");
		comm.TxCRLF();

		comm.TxSz("         ");
		ShowJct(mpdirjct[dirN]);
		comm.TxCRLF();

		comm.TxSz("     ");
		ShowJct(mpdirjct[dirW]);
		comm.TxSz(" ");
		ShowJct(jct);
		comm.TxSz(" ");
		ShowJct(mpdirjct[dirE]);
		comm.TxCRLF();

		comm.TxSz("         ");
		ShowJct(mpdirjct[dirS]);
		comm.TxCRLF();
#endif
	}
}

void MAZE::ShowJct(uchar jct) const
{
	char ch;
#ifndef WIN
	COMM comm;
#endif

	switch (jct)
	{
	case jctUnknown:
		ch = '?';
		break;
	case jctNoPath:
//		ch = 'x';
		ch = '*';
		break;
	case jctDeadEnd:
//		ch = 'X';
		ch = '#';
		break;
	case jctLoop:
		ch = 'o';
		break;
	default:
		ch = ' ';
	}

#ifdef WIN
	if (ch == ' ')
		printf("%3d", jct);
	else
		printf("%c%c%c", ch, ch, ch);
#else
	if (ch == ' ')
	{
		if (jct < 100)
		{
			comm.TxCh(' ');
			if (jct < 10)
				comm.TxCh(' ');
		}
		comm.TxNum(jct);
	}
	else
	{
//		comm.TxCh(' ');
		comm.TxCh(ch);
		comm.TxCh(ch);
		comm.TxCh(ch);
//		comm.TxCh(' ');
	}
#endif
}

#endif // DEBUG
#endif // OLD
